# -*- coding: UTF-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# cliscribe.py
#
# Librairie contenant les méthodes d'appel distant aux clients windows
#
###########################################################################

# imports twisted perspective broker
from twisted.spread import pb
from twisted.internet import reactor
# timeout
from twisted.protocols import policies
#from twisted.python import util
import sys

from manage_appli import GestAppli
from twisted.python import failure


# constantes #
port=8788
##############


class TimeoutError(Exception):
    pass

#def timeout(deferred):
#    print 'coucou'
#    return deferred.errback(failure.Failure(TimeoutError("Callback timed out")))
#
#def setmytimeout(deferred, seconds, timeoutfunc=timeout, *args, **kw):
#    deferred.mytimeout = reactor.callLater(seconds, lambda: deferred.called or timeoutfunc(deferred, *args, **kw))
#    return deferred

class Cliscribe:#(policies.TimeoutMixin):
    def __init__(self, ip, tmout=5, disconnect=True):
        self.port = port
        self.ip = ip
        self.disconnect = disconnect
        self.factory = pb.PBClientFactory()
        reactor.connectTCP(self.ip, self.port, self.factory, timeout=3)
        self.d = self.factory.getRootObject()
        # gestion du timeout
        self.setmytimeout(tmout)

    def vnc(self, ret=None, action=None, value=None, conf=None):
        # appel de la remote_fonction
        self.d.addCallback(lambda object: object.callRemote('winvnc', action, value, conf))
        self.d.addErrback(self.err)
        self.do_disconnect()
        return self.d

    def appel(self, ret=None):
        self.d.addCallback(lambda object: object.callRemote('bonjour'))
        self.d.addErrback(self.err)
        self.do_disconnect()
        return self.d

    def winreg(self, ret=None, reg_dict=None):
        self.d.addCallback(lambda object: object.callRemote('regedit', reg_dict))
        self.d.addErrback(self.err)
        self.do_disconnect()
        return self.d

    def execute(self, ret=None, cmd=None, hide=False):
        self.d.addCallback(lambda object: object.callRemote('execute', cmd, hide))
        self.d.addErrback(self.err)
        self.do_disconnect()
        return self.d

    def killproc(self, ret=None, progname=None):
        self.d.addCallback(lambda object: object.callRemote('killproc', progname))
        self.d.addErrback(self.err)
        self.do_disconnect()
        return self.d

    def shutdown(self, ret=None, bReboot=0, force=False):
        self.d.addCallback(lambda object: object.callRemote('shutdown', bReboot, force))
        self.d.addErrback(self.err)
        self.do_disconnect()
        return self.d

    def bloc(self, ret=None, partmod=None, sid=None, logon=False):
        """Ne bloque que les partages, le blocage réseau est appliqué via
        execute(cmd='wipfw drop etc...') voir manage_blocage.py
        """
        self.d.addCallback(lambda object: object.callRemote('bloc', partmod, sid, logon))
        self.d.addErrback(self.err)
        self.do_disconnect()
        return self.d

    # Fonctions de callback, errback, timeout #
    def do_disconnect(self):
        """gestion de la déconnexion
        """
        if self.disconnect:
            self.d.addCallback(self.ret)
            self.d.addErrback(self.err)
#        return self.d

    def ret(self, val=None):
        self.factory.disconnect()
#        self.factory.doStop()
        return True

    def err(self, val, ret=None):
        print 'Erreur, val:%s, ret:%s'%(val, ret)
        self.factory.disconnect()
        self.factory.doStop()
        return (val, ret)

    def setmytimeout(self, secds):
        """Deferred.setTimeout est deprecated et "it seems to work but it actually doesn't" (exarkun)
        si self.d.called != False => return self.d.called sinon exécute la fonction self.timeout
        """
        self.d.mytimeout = reactor.callLater(secds, lambda: self.d.called or self.timeout())
        return self.d.mytimeout

    def timeout(self):
        """ "fire" le deferred self.d pour passer au callback suivant (pour avoir au moins un retour dans l'appli
        """
        print 'Timeout', self.ip
        self.d.errback(failure.Failure(TimeoutError("Callback timed out")))


def rprint(resultat):
    """ 
       fonction qui se contente d'afficher le résultat
       retourné par la fonction distante
    """
    print resultat
    reactor.stop()

# appel à reserver aux tests uniquement :)
if __name__ == '__main__':
    try:
        ip = sys.argv[1]
    except:
        ip = raw_input("adresse IP du poste client : ")
    if '-killproc' in sys.argv:
        prog = sys.argv[sys.argv.index('-killproc')+1]
        print prog
        d = Cliscribe(ip, disconnect=False).killproc(progname=prog)
        d.addCallback(lambda ret: reactor.stop())
    else:
        d = Cliscribe(ip, disconnect=False).appel()
        #d.addCallback(lambda ret: reactor.stop())
        d.addCallback(rprint)
    reactor.run()





