# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# connectes.py
#
# Librairie de gestion des connectés
#
###########################################################################

import sys, commands
#sys.path.append('/usr/share/eole/backend')
from scribe.backend_conf import *

# gestion de l'annuaire
import ldap_utils    

def get_connectes(groupe=None):
    """renvoie la liste des connectes
    [PID, USERNAME, PRIMGROUP, NOMMACH, IPMACH]
    """
    # récupération des connectés
    cmd = "net status sessions"
    lignes = commands.getoutput(cmd).splitlines()[2:]
    liste=[]
    if groupe: membres = ldap_utils.get_membres(groupe)
    else: membres = None
    for ligne in lignes:
        if ligne :
            # champs : 0 pid, 1 uid, 2 grp, 3 machine, 4 (ip), 5 os (détecté)
            pid, username, prigrp, nom_mach, ip = ligne.split()
            # suppression des parenthèses
            ip = ip[1:-1]
            os = detect_os(ip)
            if membres and username not in membres: continue
            liste.append((pid, username, prigrp, nom_mach, ip, os))
    return liste

def get_groupe(ip):
    """retourne le groupe de l'utilisateur connecté sur la station "ip"
    """
    users_list = get_connectes()
    # récupération des donnees contenant ip
    donnees = get_user_from_ip(users_list, ip)
    grp = donnees[2]
    return grp

def get_user(ip):
    """retourne le username de l'utilisateur connecte sur "ip"
    """
    users_list = get_connectes()
    donnees = get_user_from_ip(users_list, ip)
    user = donnees[1]
    return user

def get_user_from_ip(liste, ip):
    """renvoie l'element dont IP=ip de la liste get_connectes
    """
    for donnee in liste:
        if ip == donnee[4] :
            return donnee
    # à l'ouverture de session XP : se connecte au domaine, se déconnecte et se reconnecte ...
    liste = file('/var/log/samba/connexions.log').read().splitlines()
    liste.reverse()
#    pid, username, prigrp, nom_mach, ip, os
#    CONNECTION Jeu 24 mai 2007 10:31 julie.evrard eleves xp-k WinXP 192.168.230.154
    for donnee in liste:
        d = donnee.split()
        pid, username, prigrp, nom_mach, ip_adr, os = '0', d[6], d[7], d[8], d[10], d[9]
        if ip_adr == ip: return (pid, username, prigrp, nom_mach, ip, os)
    raise Exception("La station %s n'est pas connectee au domaine"%ip)

def get_ip(user):
    """retourne l'ip de la station ou est connecte "user"
    """
    users_list = get_connectes()
    donnees = get_ip_from_user(users_list, user)
    ip = donnees[4]
    return ip

def get_ip_from_user(liste, user):
    """renvoie l'element dont USER=user de la liste get_connectes
    """
    for donnee in liste:
        if user == donnee[1] :
            return donnee
    raise Exception("L'utilisateur %s n'a pas de session sur le domaine"%user)

def is_connecte(user):
    connectes = [ i[1] for i in get_connectes()]
    if user in connectes: return True
    return False

def detect_os(ip):
    fich = '/var/log/samba/connexions.log'
    try:
        liste = [i.strip() for i in file(fich, 'r').readlines()]
        liste.reverse()
        for i in liste:
            cur_os, cur_ip = i.split()[9:11]
            if cur_ip == ip:
                if cur_os.lower() in ['winxp', 'win2k']:
                    return 'WinXP'
                else:
                    return 'Win9x'
        return None
    except:
        return None

def form_donnee(liste, groupe=None):
    res = []
    for donnee in liste:
        if groupe: res.append([donnee[4], donnee[1], groupe])
        else: res.append([donnee[4], donnee[1], donnee[2]])
    return res

#######################
# programme principal #
#######################
if __name__ == "__main__":
    # on peut passer la classe recherchée en argument
    try:
        groupe = sys.argv[1]
    except:
        groupe = None
    
    print groupe
    liste = get_connectes(groupe)
    liste = form_donnee(liste, groupe)
    print liste


## récupération de la classe
#def get_classe(uid):
#    """recupere la classe de "uid"
#    """
#    try:
#        query=ldap.open(serveur)
#        query.simple_bind_s("cn=admin,o=gouv,c=fr", ldap_passwd)
#        r=query.search_s("o=gouv, c=fr", ldap.SCOPE_SUBTREE, \
#                         "(uid="+uid+")")
#	#                 "(&(objectclass=Eleves)(uid="+uid+")(Divcod="+classe+"))")
#        query.unbind()
#        return r[0][1]['Divcod'][0].lower()
#    except:
#        try:
#            print r[0][1]
#        except:
#            pass	
#            return ""

#def get_connectes(groupe=None):
#    """renvoie la liste des connectes membre de 'groupe'
#    """
#    liste = tri_groupe(get_all_connectes(), groupe)
#    users = []
#    for i in liste:
#        users.append('%s:%s:%s:%s'%(i[1], i[3], i[4], i[5]))
#    users.sort()
#    return users

#def tri_groupe(donnees, groupe):
#    """renvoie les membres de 'groupe' de la liste 'liste'
#    """
#    res = []
#    for donnee in donnees:
#        groupes = ldap_utils.classe.get_groupes(
#        classe = get_classe(donnee[1])
#        if classe != '' :
#            if groupe != '' and classe != rclasse:
#                continue
#            res.append(donnee)
#    return res

