# -*- coding: UTF-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# twistd -noy controle_vnc_serveur.py
#
# Serveur Perspective Broker pour les clients Scribe/Windows
#
###########################################################################

# modules habituels
import os, sys
sys.path.append('/usr/share/eole/backend')
sys.path.append('/usr/share/eole/controlevnc')

# Perspective Broker
from twisted.spread import pb
from twisted.internet import threads, reactor

# serveur HTTP
from twisted.web import server, static

# le service
from twisted.application import service, internet
from twisted.python import log

# gestion de l'annuaire
import ldap_utils

# gestion des clients
from cliscribe import Cliscribe
import connectes
import gest_sessions

# gestion des stations
import manage_stations

# gestion des applications
from manage_appli import GestAppli

# gestion des blocage réseau
from manage_blocage import GestBlocage

# gestion des devoirs
import devoirs

# autorisations d'accès (basé sur IP ...)
import autorisations

#from mytest import Cliscribe


# gestion des applications => à DéGAGER
def get_apps(ip):
    """retourne le dictionnaire d'applications autorisées
    pour le groupe de l'utilisateur connecté sur IP
    """
    try:
        grp = connectes.get_groupe(ip)
        ga = GestAppli()
        return ga.get_applis(grp)
    except:
        return None


######################
#  appels distants   #
######################
class Echoer(pb.Root):
    """Classe contenant les fonctions distantes "remote_<func>" accessibles depuis les postes clients.
    Protection de certaines en fonction du groupe dont fait parti le login qui a ouvert une session Samba
    sur la station tentant d'accéder à la remote_fonction.
    """
    def __init__(self):
        self.bloc = GestBlocage()
        return
    
#    def rootObject(self, broker):
#        """récupération d'informations sur la connection (ex: IP)
#        """
#        self.p_broker = broker
#        return pb.Root.rootObject(self, broker)

    def remoteMessageReceived(self, broker, message, args, kw):
        #self.ip = self.p_broker.transport.getPeer().host
        self.ip = broker.transport.getPeer().host
        return pb.Root.remoteMessageReceived(self, broker, message, args, kw)

    #######################
    # Fonctions distantes #
    #######################
    
    def remote_bonjour(self, ip, ret=None):
        """Appel de la fonction bonjour sur client B par client A
        """
        if not autorisations.isprof(self.ip):
            self.p_broker.transport.loseConnection()
            log.msg('Appel non autorisé (non prof) de remote_bonjour par %s'%self.ip)
            return
        log.msg('Appel de la fonction remote_bonjour par %s sur %s'%(self.ip, ip))
        d = Cliscribe(ip).appel()
        return d

    # Listes des groupes, des connectes et des membres de groupes #
    def remote_connectes(self, classe=''):
        """gestion controle-vnc
        """
        log.msg('Appel de la fonction remote_connectes (classe=%s) par %s'%(classe, self.ip))
        return connectes.get_connectes(classe)
    
    def remote_groupes(self):
        """renvoie la liste des groupes
        """
        log.msg('Appel de la fonction remote_groupes par %s'%self.ip)
        return ldap_utils.get_groupes()

    def remote_classes(self):
        """renvoie la liste des classes
        """
        log.msg('Appel de la fonction remote_classes par %s'%self.ip)
        return ldap_utils.get_classes()
    
    def remote_membres(self, groupe):
        """renvoie la liste des membres de 'groupe'
        """
        log.msg('Appel de la fonction remote_membres par %s'%self.ip)
        return ldap_utils.get_membres(groupe)
    
    # Observation VNC #
    def remote_get_vnc_mode(self):
        log.msg('Appel de la fonction remote_get_vnc_mode par %s'%(self.ip))
        return gest_sessions.Logon().get_vnc_mode()

    def remote_observe(self, cible, active=None):
        """src est en mode "listen"
        clibe "connect" src, sur cible : 'winvnc.exe -connect src (self.ip)'
        """
        if not autorisations.isprof(self.ip):
            self.p_broker.transport.loseConnection()
            log.msg('Appel non autorisé (non prof) de remote_observe par %s'%self.ip)
            return
        log.msg('Appel de la fonction remote_observe par %s sur %s (active = %s)'%(self.ip, cible, active))
        # force le mode d'observation fixé dans l'EAD
        if not autorisations.isadmin(self.ip) and gest_sessions.Logon().get_vnc_mode() == 'simple': active=None
        d = Cliscribe(cible).vnc(action='setinputs', value=active)
        d.addCallback(Cliscribe(cible).vnc, action='connect', value=self.ip)
        return d
        
    # Diffusion VNC #
    def remote_diffusion(self, ip_liste):
        """active le mode vncviewer /listen sur les ips de ip_liste
        """
        if not autorisations.isprof(self.ip):
            self.p_broker.transport.loseConnection()
            log.msg('Appel non autorisé (non prof) de remote_diffusion par %s'%self.ip)
            return
        log.msg('Appel de la fonction remote_diffusion par %s sur %s'%(self.ip, ip_liste))
        d = Cliscribe(self.ip).vnc(action='setinputs')
        # fichier généré par vncviewer.exe configuration en 256 couleurs
        myconf = file('/home/workgroups/professeurs/gestion-postes/eole_conf.vnc').read()
        for cible in ip_liste:
            d.addCallback(Cliscribe(cible).vnc, action='start_listen', conf=myconf)
        return d

    # Blocage Internet #
    def remote_get_bloc_list(self):
        """Renvoie la liste des type de blocages disponibles (Aucun=0, Tout internet=1, ...)
        """
        if not autorisations.isprof(self.ip):
            self.p_broker.transport.loseConnection()
            log.msg('Appel non autorisé (non prof) de remote_get_bloc_list par %s'%self.ip)
            return
        log.msg('Appel de la fonction remote_get_bloc_list par %s'%self.ip)
        return self.bloc.get_bloc_list()

    def remote_users_bloc_mod(self, groupe):
        """renvoie un dictionnaire des membres du groupe avec leur mode de blocage
        { user1 : {'netmod' : 1, 'partmod' : 0}, user2 :  ... }
        """
        if not autorisations.isprof(self.ip):
            self.p_broker.transport.loseConnection()
            log.msg('Appel non autorisé (non prof) de remote_users_bloc_mod par %s'%self.ip)
            return
        log.msg('Appel de la fonction remote_users_bloc_mod par %s'%self.ip)
        users = ldap_utils.get_membres(groupe)
        dico = dict()
        for user in users:
            dico[user] = self.bloc.get_blocage(user)
        return dico

    def remote_set_user_bloc_mod(self, users):
        """Applique le blocage demandé aux utilisateurs sélectionnés
        users = [(user, netmod, partmod, durée du blocage), (user2, ...)]
        """
        if not autorisations.isprof(self.ip):
            self.p_broker.transport.loseConnection()
            log.msg('Appel non autorisé (non prof) de remote_set_users_bloc_mod par %s'%self.ip)
            return
        log.msg('Appel de la fonction remote_set_user_bloc_mod par %s'%self.ip)
        for i in users:
            user, netmod, partmod, duree = i
            self.bloc.set_blocage(user, netmod, partmod, duree)
        return True

    def remote_get_own_bloc_mod(self):
        log.msg('Appel de la fonction remote_get_own_bloc_mod par %s'%self.ip)
        return self.bloc.get_blocage(connectes.get_user(self.ip))

    # Distribution de devoirs
    def remote_nb_membres(self, groupe):
        """Renvoie le nombre de membres du groupe
        """
        log.msg('Appel de la fonction remote_nb_membres par %s pour %s'%(self.ip, groupe))
        return ldap_utils.get_nb_membres(groupe)

    def remote_distribute(self, username, groupe, dev_name):
        """distribue le devoir "dev_name" du prof "username" à "groupe"
        """
        if not autorisations.isprof(self.ip):
            self.p_broker.transport.loseConnection()
            log.msg('Appel non autorisé (non prof) de remote_distribute par %s'%self.ip)
            return
        log.msg('Appel de la fonction remote_distribute par %s : username=%s, groupe=%s, dev_name=%s'%(self.ip, username, groupe, dev_name))
        return devoirs.distribute(username, groupe, dev_name)

    def remote_ramasse(self, username, groupe, dev_name):
        """rammasse "dev_name"
        """
        if not autorisations.isprof(self.ip):
            self.p_broker.transport.loseConnection()
            log.msg('Appel non autorisé (non prof) de remote_ramasse par %s'%self.ip)
            return
        log.msg('Appel de la fonction remote_ramasse par %s : username=%s, groupe=%s, dev_name=%s'%(self.ip, username, groupe, dev_name))
        return devoirs.ramasse(username, groupe, dev_name)

    def remote_rendre(self, username, groupe, dev_name):
        """rend la correction de "dev_name"
        """
        if not autorisations.isprof(self.ip):
            self.p_broker.transport.loseConnection()
            log.msg('Appel non autorisé (non prof) de remote_rendre par %s'%self.ip)
            return
        log.msg('Appel de la fonction remote_rendre par %s : username=%s, groupe=%s, dev_name=%s'%(self.ip, username, groupe, dev_name))
        return devoirs.rendre(username, groupe, dev_name)

    def remote_supprimer(self, username, groupe, dev_name):
        """Supprime dev_name de la liste des devoirs et supprime les données correspondantes
        (sinon l'élève ne peut supprimer le dossier du devoir dans son rep perso
        """
        if not autorisations.isprof(self.ip):
            self.p_broker.transport.loseConnection()
            log.msg('Appel non autorisé (non prof) de remote_supprimer par %s'%self.ip)
            return
        log.msg('Appel de la fonction remote_supprimer par %s : username=%s, dev_name=%s'%(self.ip, username, dev_name))
        return devoirs.supprimer(username, groupe, dev_name)

    # Gestion des postes (démarrage) #
    def remote_service_start(self, nom):
        """gestion des postes allumés
        """
        log.msg('Appel de la fonction remote_service_start par %s : %s'%(self.ip, nom))
        return manage_stations.add_sta(self.ip, nom)
    
    # Gestion des postes (arrêt) #
    def remote_service_stop(self):
        log.msg('Appel de la fonction remote_service_stop par %s'%self.ip)
        return manage_stations.del_sta(self.ip)

#    # Gestion des applications #
#    def remote_get_apps(self):
#        """renvoie la listes des applications
#        en fonction du groupe de l'utilisateur
#        """
#        log.msg('Appel de la fonction remote_get_apps par %s'%self.ip)
#        return get_apps(self.ip)
#
#    def remote_inst_app(self, app, ip=None):
#        """Installation d'applications
#        """
#        if not autorisations.isprof(self.ip):
#            self.p_broker.transport.loseConnection()
#            log.msg('Appel non autorisé (non prof) de remote_supprimer par %s'%self.ip)
#            return
#        if not ip: ip = self.ip
#        log.msg('Appel de la fonction remote_inst_app "%s" par %s'%(app, self.ip))
#        grp = connectes.get_groupe(self.ip)
#        inst = cliscribe.InstApp(self.ip, app, grp)
#        inst.start()
#        return inst.d

    # Gestion de sessions cliente #
    # Ouverture de session
    def remote_logon(self, os_type):
        """fonction appelée par les clients lors de l'ouverture de session
        (dés)active le blocage
        """
        log.msg('Appel de la fonction remote_logon par %s'%(self.ip))
        gest_sessions.Logon().logon(self.ip, os_type, self.bloc)
        return True



#######################
# programme principal #
#######################
if __name__ == '__main__':
    sys.exit("Usage : twistd -noy %s"%os.path.basename(sys.argv[0]))

# Après un arrêt de controle-vnc, purge des stations allumees
manage_stations.purge()
application = service.Application('ControleVNCserveur')
# serveur de téléchargement de client Scribe
root = static.File('/home/client_scribe')
site = server.Site(root)
log.msg('Controle VNC Serveur - Fichiers demarre port:8790')
internet.TCPServer(8790, site).setServiceParent(service.IServiceCollection(application))
# serveur PB pour exécution de fonctions distantes
pbfactory = pb.PBServerFactory(Echoer())
log.msg('Controle VNC Serveur demarre port:8789')
internet.TCPServer(8789, pbfactory).setServiceParent(service.IServiceCollection(application))

