# -*- coding: UTF-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# devoirs.py
#
# Librairie de gestion de la distribution de devoirs
#
###########################################################################
 

import sys, os, shutil
import ldap_utils

sys.path.append('/usr/share/eole/controlevnc')


def distribute(username, groupe, dev_name):
    """distribue /home/<u>/<username>/devoirs/<dev_name>/ aux membres de <groupe>
    """
    user_liste = ldap_utils.get_membres(groupe)
    rep_path = os.path.join('/home', 'workgroups', 'devoirs', username, dev_name)
    donnees = os.path.join(rep_path, 'donnees')
    os.system('/usr/bin/setfacl -m g:%s:r-x %s'%(groupe, donnees))
    for user in user_liste:
        user_dev_dir = os.path.join('/home', user[0], user, 'perso', 'devoirs')
        if not os.path.exists(user_dev_dir): os.makedirs(user_dev_dir)
        user_dest_dir = os.path.join(user_dev_dir, dev_name)
        copy_dev(rep_path, user_dest_dir)
#    shutil.rmtree(rep_path)
    return len(user_liste)

def copy_dev(rep_path, user_dest_dir):
    """copie <rep_path>/* dans /home/<u>/<user>/perso/devoirs/<dev_name>
    crée un lien /home/<u>/<user>/perso/devoirs/<dev_name>/donnees => <rep_path>/donnees
    """
    if os.path.exists(user_dest_dir): shutil.rmtree(user_dest_dir)
    os.makedirs(user_dest_dir)
    for i in os.listdir(rep_path):
        f = os.path.join(rep_path, i)
        if os.path.isfile(f): shutil.copy(f, user_dest_dir)
    if os.path.isdir(os.path.join(rep_path, 'donnees')):
        os.symlink(os.path.join(rep_path, 'donnees'), os.path.join(user_dest_dir, 'donnees'))

def ramasse(username, groupe, dev_name):
    user_liste = ldap_utils.get_membres(groupe)
    dest = os.path.join('/home', username[0], username, 'perso', 'devoirs', '%s-%s'%(dev_name, groupe))
    if not os.path.exists(dest): os.makedirs(dest)
    rate, reussi= 0, 0 
    for user in user_liste:
        src = os.path.join('/home', user[0], user, 'perso', 'devoirs', dev_name)
        dst = os.path.join(dest, user)
        if os.path.exists(dst): shutil.rmtree(dst)
        os.makedirs(dst)
        try:
            for i in os.listdir(src):
                if os.path.isfile(os.path.join(src, i)): shutil.copy(os.path.join(src, i), dst)
            reussi += 1
        except Exception, e:
            rate += 1
            print 'Impossible de ramasser le devoir de %s : %s'%(user, e)
    return (reussi, rate)

def rendre(username, groupe, dev_name):
    source = os.path.join('/home', username[0], username, 'perso', 'devoirs', '%s-%s'%(dev_name, groupe))
    rate, reussi= 0, 0 
    for user in os.listdir(source):
        src = os.path.join(source, user)
        dst = os.path.join('/home', user[0], user, 'perso', 'devoirs', dev_name, 'correction')
        if os.path.exists(dst): shutil.rmtree(dst)
        os.makedirs(dst)
        try:
            for i in os.listdir(src): shutil.copy(os.path.join(src, i), dst)
            reussi += 1
        except Exception, e:
            rate += 1
            print 'Impossible de rendre le devoir à %s : %s'%(user, e)
    return (reussi, rate)

def supprimer(username, groupe, dev_name):
    for user in ldap_utils.get_membres(groupe):
        cible = os.path.join('/home', user[0], user, 'perso', 'devoirs', dev_name, 'donnees')
        if os.path.islink(cible): os.remove(cible)
    cible = os.path.join('/home', 'workgroups', 'devoirs', username, dev_name)
    if os.path.isdir(cible): shutil.rmtree(cible)
    return True

