# -*- coding: UTF-8 -*-
from os.path import join as path_join
from xml.etree import ElementTree as ET
from connectes import get_nom_machine, detect_os
from fnmatch import fnmatch
XMLDIR = "/home/esu/Base"

OS_XP = 252

WINOS = {
    'Win95': 128,  # Windows 95
  # 'Win98': 64,   # Windows 98
  # 'WinMe': 32,   # Windows Me
    'WinNT': 16,   # Windows NT4
    'Win2K': 8,    # Windows 2000
    'WinXP': 4,    # Windows XP
 #   'Win2K3': ???, # Windows 2003
 #   'WinVista': ???, # Windows Vista
}

REG_TYPES = {
    'DWORD': 'REG_DWORD',
    'STRING': 'REG_SZ',
    'BOOLEAN': 'REG_DWORD',
}

class ESU:
    def __init__(self, logon):
        self._prefs_js = []
        self._regedit = []
        self._inifile = []
        self._privilege = []
        self._acl_fichier = []
        self._acl_registre = []
        self.logon = logon

        self.sid = "S-1-5-21-2589691161-2360211008-958261667-21008"
        ip = "192.168.230.154"
        groupes = ('eleves', 'omainAdmins', 'professeurs', 'marcel')

#        self.sid = self.logon.sid
#        ip = self.logon.ip
#        groupes = self.logon.groups

        nom_machine = get_nom_machine(ip)
        os = detect_os(ip)
        self.os = WINOS[os]

        # Charge les options de la machine
        filename = path_join(XMLDIR, "ListeGM.xml")
        dirname = self.searchComputer(filename, nom_machine)
        dirname = path_join(XMLDIR, dirname)
        machine_filename = path_join(dirname, "_Machine.xml")

        # Charge les options selon le groupe
        user_filename = self.searchGroup(dirname, groupes)
        if not user_filename:
            # TODO: disconnect user
            return
        self.loadConfig(machine_filename)
        self._regedit_machine = self._regedit
        self._regedit = []
        self.loadConfig(user_filename)
        self._regedit_user = self._regedit
        # TODO: Use self._regedit, self._prefs_js, self._inifile

    def loadConfig(self, filename):
        """
        Charge des regles depuis un fichier XML.
        """
        xml = ET.parse(filename)
        for chemin in xml.find('Regles').getiterator('Chemin'):
            nom = chemin.get('nom')
            for variable in chemin.getiterator('Variable'):
                self.readRule(nom, variable)

    def readRule(self, nom, variable):
        # Filtre sur la version de l'OS
        os = int(variable.get('OS'))
        if not (os & self.os):
            return
        varnom = variable.get('nom')
        vartype = variable.get('type')
        value = variable.findtext('.')
        if nom.startswith("REG://"):
            self.regedit("%s\\%s" % (nom[6:], varnom), vartype, value)
        elif nom.startswith("INI://"):
            filename, section = nom[6:].split("?section=", 1)
            self.setini(filename, section, varnom, value, vartype)
        elif nom.startswith("JS://"):
            self.setjs(nom[5:], varnom, value, vartype)
        elif nom == "ACLREGISTRE":
            self.aclreg(varnom, value)
        elif nom == "ACLFICHIER":
            self.aclfile(varnom, value)
        elif nom == "PRIVILEGE":
            self.privilege(varnom, value)
        else:
            print "UNKNOWN ACTION", nom

    def regedit(self, key, keytype, value):
        keytype = REG_TYPES[keytype]

        # a\b\c\d\e => a, b\c\d, e
        key = key.rstrip("\\")
        hkey, keypath = key.split("\\", 1)
        if hkey == "HKEY_CURRENT_USER":
            hkey = "HKEY_USERS"
            keypath = "%s\\%s" % (self.sid, keypath)
        keypath, keyname = keypath.rsplit("\\", 1)

        reg = {
            'hkey': hkey,
            'keypath': keypath,
            'keyname': keyname,
            'keytype': keytype,
            'value': value,
        }
        self._regedit.append(reg)

    def aclfile(self, filename, content):
        self._acl_fichier.append((filename, content))

    def aclreg(self, reg, value):
        self._acl_registre.append((reg, value))

    def privilege(self, key, content):
        self._privilege.append((key, content))
        return
        print "PRIVILEGE: %s=%r" % (key, content)

    def setini(self, filename, section, key, value, valtype):
        self._inifile.append( (filename, section, key, value, valtype) )

    def setjs(self, filename, key, value, valtype):
        self._prefs_js.append( (filename, key, value, valtype) )

    def searchComputer(self, filename, nom_machine):
        """
        Cherche le groupe qui matche le nom de la machine.
        Caractères spéciaux pour matcher le nom : *, ?, [ et ].
        Voir http://docs.python.org/lib/module-fnmatch.html

        Utilise le fichier XML "ListeGM.xml".
        """
        xml = ET.parse(filename)
        for group in xml.getiterator('GM'):
            motif = group.find('ListeMachines').findtext("Machine")
            if fnmatch(nom_machine, motif):
                return group.get('nom')
        raise LookupError("Impossible de trouver le groupe correspondant la machine %s" % nom_machine)

    def searchGroup(self, dirname, groupes):
        """
        Cherche le nom du fichier XML selon les groupes de l'utilisateur
        """
        filename = path_join(dirname, "_ListeUtilisateurs.xml")
        xml = ET.parse(filename)
        for groupe in xml.getiterator('GU'):
            nom = groupe.get('nom')
            if nom in groupes:
                return path_join(dirname, "%s.xml" % nom)
        return None

def main():
    ESU(None)

if __name__ == "__main__":
    main()

