#!/usr/bin/python
# -*- coding: UTF-8 -*-
import re

STRING_REGEX = r'"(?:[^"\\]|\\")*"'
regex = r"user_pref\s*\((%s)\s*,\s*(%s|[^ )]+)\s*\)" % (
    STRING_REGEX, STRING_REGEX)
USER_PREF_REGEX = re.compile(regex, re.MULTILINE)

class FirefoxPrefsJS:
    def __init__(self, filename):
        """
        Charge les préferrences depuis un fichier "prefs.js"
        """
        self.filename = filename
        self.prefs = {}
        prefs = open(filename).read()
        for key, value in USER_PREF_REGEX.findall(prefs):
            self.prefs[key] = value

    def __setitem__(self, key, value):
        """
        Modifie la valeur d'une préferrence où key est une clé sans
        guillemets, ex: "browser.download.lastDir"
        """
        key = '"%s"' % key
        self.prefs[key] = value

    def __delitem__(self, key):
        """
        Supprime une entrée où key est une clé sans
        guillemets, ex: "browser.download.lastDir".

        Ne génère pas d'erreur si la clé n'existait pas.
        """
        key = '"%s"' % key
        try:
            del self.prefs[key]
        except KeyError:
            pass

    def save(self, filename):
        """
        Sauvegarde la confirmation dans le fichier spécifié.
        Écrit des lignes de la forme:
           user_pref("cle", valeur);
        """
        content = []
        for key, value in self.prefs.iteritems():
            content.append("user_pref(%s, %s);" % (key, value))
        open(filename, 'w').write("\n".join(content))
        print "Wrote %s" % filename

def main():
    prefs = FirefoxPrefsJS("prefs.js")
    prefs["browser.download.lastDir"] = '"/tmp/download"'
    del prefs["accessibility.typeaheadfind.flashBar"]
    del prefs["accessibility.typeaheadfind.flashBar"]
    prefs.save("new-prefs.js")

if __name__ == "__main__":
    main()

