# -*- coding: UTF-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# ldap_utils.py
#
# Librairie pour la gestion des requetes LDAP
#
###########################################################################
 

import sys, ldap

sys.path.append('/usr/share/eole/backend')
sys.path.append('/usr/share/eole/controlevnc')

from creole import parsedico


class Ldap:
    def __init__(self, serveur='127.0.0.1'):
        self.query = ldap.open(serveur)
        self.query.simple_bind("cn=admin,o=gouv,c=fr")

    def unbind(self):
        self.query.unbind()

    def get_classes(self):
        """renvoie la liste des classes
        """
        result_group = self.query.search_s("o=gouv,c=fr", 2, "(&(objectclass=posixGroup)(description=Classe *))")
        # on créé la liste des noms de groupes
        liste_group = []
        for groupe in result_group:
            try: liste_group.append(groupe[1]['cn'][0])
            except: continue
        liste_group.sort()
        return liste_group
    
    def get_groupe_liste(self):
        """renvoie la liste des groupes
        """
        result_dispo = self.query.search_s("o=gouv,c=fr", 2, "(objectclass=posixGroup)")
        return result_dispo
    
    def get_groupes(self, banned=True):
        """renvoie la liste des groupes
        exceptés DomainAdmins, DomainComputers, DomainUsers, PrinterAdmins, PrintOperators
        """
        banned_groups = ['DomainAdmins', 'DomainComputers', 'DomainUsers', 'PrinterAdmins', 'PrintOperators']
        result_dispo = self.get_groupe_liste()
        liste_group = []
        for group in result_dispo:
            gname = group[1]['cn'][0]
            if banned and gname in banned_groups: continue
            liste_group.append(gname)
        liste_group.sort()
        return liste_group
    
    def get_membres(self, groupe):
        result_group = self.query.search_s("o=gouv,c=fr", 2, "(&(cn=%s))"%groupe)
        if len(result_group) == 0: raise Exception('Groupe %s non trouve'%groupe)
        # on créé la liste des noms de groupes
        liste_membres = result_group[0][1]['memberUid']
        liste_membres.sort()
        return liste_membres
    
    def get_nb_membres(self, groupe):
        return len(self.get_membres(groupe))
    
    def get_user_groups(self, user, assoc=True, banned=True):
        """récupère la liste des groupes dont 'user' fait partie si assoc=True
        sinon renvoie la liste des autres groupes
        la liste est filtrée sur bannedgrps si banned
        """
        result_dispo = self.get_groupe_liste()
        bannedgrps = ['DomainComputers']
        liste_group = []
        for group in result_dispo:
            try: gname = group[1]['cn'][0]
            except: continue
            
            try: members = group[1]['memberUid']
            except: members = []
    
            if gname in bannedgrps and banned: continue
            if assoc:
                if user in members:
                    liste_group.append(gname)
            else:
                if user not in members:
                    liste_group.append(gname)
        liste_group.sort()
        return liste_group
    
    def get_user_shares(self, user):
        """ fonction de récupération des partages associés à un utilisateur
        """
        groups = self.get_user_groups(user)
        shares = self.get_shares()
        share_user=[]
        for share in shares:
            sharepath, sharegroup, sharename = share
            if sharegroup in groups:
                share_user.append((sharename,sharepath))
        return share_user
    
    def get_user_homedrive(self, user):
        """récupération du chemin du répertoire perso d'un utilisateur
        """
        result_user = self.query.search_s("o=gouv,c=fr", 2,\
                "(&(objectclass=sambaSAMAccount)(objectclass=inetOrgPerson)(uid=%s))"%user)
        # création de la liste des noms récupérés
        dicoEole = parsedico.parse_dico()
        return (str(result_user[0][1]['sambaHomeDrive'][0]),"\\\\%s\\PERSO"%dicoEole["smb_netbios_name"])
    
    def get_shares(self):
        """liste de tous les répertoires partagés
        """
        result_share = self.query.search_s("o=gouv,c=fr", 2, "(objectclass=sambaFileShare)")
        list_share = []
        for share in result_share:
            list_share.append((share[1]['sambaShareURI'][0], share[1]['sambaShareGroup'][0], share[1]['sambaShareName'][0],))
        return list_share
    
    def get_user_sid(self, user):
        """récupération du sid de l'utilisateur
        """
        try: sid = self.query.search_s("o=gouv,c=fr", 2, "(&(objectclass=sambaSAMAccount)(uid=%s))"%user)[0][1]['sambaSID'][0]
        except: raise Exception('Utilisateur %s non trouve'%user)
        return sid


def get_classes():
    l = Ldap()
    r = l.get_classes()
    l.unbind()
    return r

def get_groupes():
    l = Ldap()
    r = l.get_groupes()
    l.unbind()
    return r

def get_shares():
    l = Ldap()
    r = l.get_shares()
    l.unbind()
    return r

def get_membres(groupe):
    l = Ldap()
    r = l.get_membres(groupe)
    l.unbind()
    return r

def get_nb_membres(groupe):
    l = Ldap()
    r = l.get_nb_membres(groupe)
    l.unbind()
    return r

def get_user_groups(user):
    l = Ldap()
    r = l.get_user_groups(user)
    l.unbind()
    return r

def get_user_shares(user):
    l = Ldap()
    r = l.get_user_shares(user)
    l.unbind()
    return r

def get_user_homedrive(user):
    l = Ldap()
    r = l.get_user_homedrive(user)
    l.unbind()
    return r

def get_user_sid(user):
    l = Ldap()
    r = l.get_user_sid(user)
    l.unbind()
    return r


