# -*- coding: UTF-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# manage_appli.py
#
# Librairie de gestion d'installation d'applications sur les clients Windows
#
###########################################################################

import pickle
import sys
import os
import shutil
import md5

from ConfigParser import ConfigParser
from getopt import getopt, GetoptError


app_path = '/home/client_scribe/applications/'
f_appli = os.path.join(app_path, 'applications.cfg')

class GestAppli:
    """Gestion des applications à installer sur les postes client.
    Utilisation de cliscribe pour les installations/lancement.
    """

    def __init__(self):
        self.f_appli = f_appli

    def get_applis(self, grp):
        """retourne un dictionnaire contenant les application dispo pour le groupe "grp"
        dico = { appli : { 'titre' : 'titre application'}, appli2 : ...}
        """
        dico = {}
        for app in self.get_apps():
            if grp in self.get_grps(app):
                dico[app] = self.get_app_dict(app, grp, short=True)
        return dico

    def get_app_dict(self, app, grp, short=False):
        """retourne un dictionnaire contenant les détails de "app"
        app_dict = { 'nom':nom, 'titre':titre, 'exe':exe, 'args':args, 'groupes':groupes, 'exe_path':path_vers_exe}
        """
        
        if grp.lower() not in self.get_grps(app): return None
        app_dict = dict(nom = app)
        app_dict['titre'] = self.get_titre(app)
        if short: return app_dict
        app_dict['cmd'] = self.get_cmd(app)
        app_dict['exe'] = self.get_exe(app)
        app_dict['args'] = self.get_args(app)
        app_dict['groupes'] = self.get_grps(app)
        app_dict['exe_path'] = os.path.join(app, app_dict['exe'])
        return app_dict
    
    def get_apps(self):
        """retourne la liste des applications
        """
        c = ConfigParser()
        c.read(self.f_appli)
        return c.sections()

    def get_value(self, app, name):
        """retourne la valeur de "name" pour "app"
        """
        app = str(app)
        try:
            c = ConfigParser()
            c.read(self.f_appli)
            value  = c.get(app, name)
            value = unicode(value, 'iso-8859-1')
        except:
            value = None
        return value

    def get_titre(self, app):
        """retourne le titre de l'application "app"
        """
        titre = self.get_value(app, 'titre')
        if not titre: titre = app
        return titre

    def get_exe(self, app):
        """retourne le nom de l'executable de l'application
        ou None
        """
        exe = self.get_value(app, 'exe')
        return exe
    
    def get_cmd(self, app):
        """retourne la commande msiexe de l'application
        ou None
        """
        cmd = self.get_value(app, 'cmd')
        return cmd
    
    def get_args(self, app):
        """retourne la commande msiexe de l'application
        ou None
        """
        args = self.get_value(app, 'args')
        return args
    
    def get_nom(self, app):
        """retourne la valeur de "id" de l'application
        ou None
        """
        nom = self.get_value(app, 'nom')
        return nom
    
    def get_grps(self, app):
        """retourne la liste des groupes autorisés à utiliser
        l'application
        """
        grps = eval(self.get_value(app, 'groupes'))
        lgrps = [ i.lower() for i in grps ]
        return lgrps

    def inst_app(self, app, ip):
        """installe "app" sur "ip"
        """
        exe = self.get_exe(app)
        return

def param_appli(appli):
    """liste les paramètres de l'application "appli"
    """
    c = ConfigParser()
    c.read(f_appli)
    res = ''
    if appli_exist(appli):
        for i in c.items(appli):
            res += '%s:%s;'%i
        print res[:-1]
        return True
    return False

def appli_exist(appli):
    """test si une application existe déjà
    """
    c = ConfigParser()
    c.read(f_appli)
    return c.has_section(appli)

def liste_appli():
    """Liste les applications diponibles
    """
    c = ConfigParser()
    c.read(f_appli)
    for i in c.sections(): print i
    return True

def del_appli(appli):
    """supprime l'application "appli"
    - supprime le dossier et les fichiers (paquet + md5sum)
    - supprime l'entrée dans le fichier de configuration "applications.cfg"
    """
    dst = os.path.join(app_path, appli)
    os.system('rm -rf %s'%dst)
    c = ConfigParser()
    c.read(f_appli)
    c.remove_section(appli)
    c.write(file(f_appli, 'w'))
    return True

def add_appli(nom, titre, exe, cmd, args, groupes):
    """Ajoute l'application
    - copie le fichier
    - calcule la somme MD5
    - ajoute l'entrée dans le fichier de configuration "applications.cfg"
    """
    exe_name = os.path.basename(exe)
    if not cmd: cmd = ''
    dst = os.path.join(app_path, nom)
    if os.path.isdir(dst):
        sys.exit("L'application %s existe déjà"%nom)
    os.makedirs(dst)
    os.system('chown root:root %s'%exe)
    shutil.move(exe, dst)
    dexe = os.path.join(dst, exe_name)
    md5sum = md5.md5(file(dexe,'rb').read()).hexdigest().strip()
    md5fich = os.path.join(dst, '%s.MD5SUM'%exe_name)
    file(md5fich, 'w').write(md5sum)
    gen_app_dict(nom, titre, exe_name, cmd, args, groupes)
    return True

def gen_app_dict(nom, titre, exe, cmd, args, groupes):
    """ajoute la configuration de l'application dans le fichier "applications.cfg"
    """
    c = ConfigParser()
    c.read(f_appli)
    if not appli_exist(nom):
        c.add_section(nom)
        c.set(nom, 'exe', exe)
        c.set(nom, 'titre', titre)
        c.set(nom, 'cmd', cmd)
        c.set(nom, 'args', args)
        c.set(nom, 'groupes', groupes.split(','))
    else:
        if exe: c.set(nom, 'exe', exe)
        if titre: c.set(nom, 'titre', titre)
        if cmd: c.set(nom, 'cmd', cmd)
        if args: c.set(nom, 'args', args)
        if groupes: c.set(nom, 'groupes', groupes.split(','))
    c.write(file(f_appli, 'w'))
    return True

def supp_spec_chars(chaine):
    """remplace tous les caractères qui ne sont pas alphanum (sans accents) par _
    """
    liste = range(48, 58)
    liste.extend(range(65, 91))
    liste.extend(range(97, 123))
    for i in chaine:
        if ord(i) not in liste:
            chaine = chaine.replace(i, '_')
    return chaine

def help():
    dico = dict(app = os.path.basename(sys.argv[0]))
    msg="""
%(app)s -h --help (affiche ce message)

Paramètre nécessaires à %(app)s :
    -n --nom (nom de l'application)
    -t --titre (titre de l'application)
    -e --exe (chemin vers le .msi/.exe)
    -c --cmd (commande pour exécuter le .msi ex: msiexec /i, ne pas renseigner pour un .exe)
    -a --args (arguments à ajouter à la commande ex: /qn pour une installation silencieuse du .msi)
    -g --groupes (liste des groupes autorisés à installer cette application, séparateur ",")

    -d --delete APPLI (supprime APPLI)
    
    -l --liste (liste les applications disponibles)
    -p --params APPLI (récupère les paramètres de l'application)
    """%dico
    sys.exit(msg)

def main():
    nom, titre, exe, cmd, args, groupes, delete, liste, param = None, None, None, None, None, None, None, None, None
    try:
        options, args = getopt(sys.argv[1:],'n:t:e:c:a:g:d:p:lh',
        ['nom=',
        'titre=',
        'exe=',
        'cmd=',
        'args=',
        'groupes=',
        'delete=',
        'params=',
        'liste',
        'help'])

    except GetoptError:
        er_type, er_value, er_traceback = sys.exc_info()
        er_msg = str(er_value)
        sys.exit("l'option -%s est inconnue ou nécessite un argument" % er_msg.split(' ')[1])

    for (opt,val) in options:
        if (opt == '-n') or (opt == '--nom'):
            nom = val.lower()
        elif (opt == '-t') or (opt == '--titre'):
            titre = val.lower()
        elif (opt == '-e') or (opt == '--exe'):
            exe = val.lower()
        elif (opt == '-c') or (opt == '--cmd'):
            cmd = val.lower()
        elif (opt == '-a') or (opt == '--args'):
            args = val.lower()
        elif (opt == '-g') or (opt == '--groupes'):
            groupes = val.lower()
        elif (opt == '-d') or (opt == '--delete'):
            delete = val.lower()
        elif (opt == '-l') or (opt == '--liste'):
            liste = True
        elif (opt == '-p') or (opt == '--params'):
            param = val.lower()
        elif (opt == '-h') or (opt == '--help'):
            help()
            return
    
    # ajout / modification d'appli
    if nom:
        # ajout
        if exe:
            if titre and groupes:
                nom = supp_spec_chars(nom)
                if not args: args=''
                if os.path.splitext(exe)[1] == '.msi' and not cmd: cmd='msiexec /i'
#                    sys.exit('Erreur : Il faut spécifier une commande pour les paquets .msi')
                elif not cmd: cmd = ''
            else:
                sys.exit('Il manque une option "titre" / "groupes"')
            add_appli(nom, titre, exe, cmd, args, groupes)
        else:
            gen_app_dict(nom, titre, exe, cmd, args, groupes)
        return
    # suppression d'appli
    elif delete:
        del_appli(delete)
        return
    # listing
    elif liste:
        liste_appli()
        return
    # affichage des paramètres d'une appli
    elif param:
        param_appli(param)
        return
    help()

if __name__ == '__main__':
    main()

